use actix_web::{web, HttpResponse, Responder};
use uuid::Uuid;
use chrono::Local;
use log::{info, error, debug};

use crate::models::{CreateDesignRequest, CreateDesignResponse, Design};
use crate::database::Database;
use crate::external_image_generator::ExternalImageGenerator;

pub async fn create_design(
    db: web::Data<Database>,
    image_generator: web::Data<ExternalImageGenerator>,
    req: web::Json<CreateDesignRequest>,
) -> impl Responder {
    let id = Uuid::new_v4();
    info!("Processing create_design request for user: {}", req.user_id);
    debug!("Design details - Style: {}, Prompt: {}", req.style, req.prompt);
    
    let design = Design {
        id,
        style: req.style.clone(),
        prompt: req.prompt.clone(),
        image: "".to_string(),
        created_at: Local::now().naive_local(),
        user_id: req.user_id.clone(),
    };

    let base64_image = match web::block(move || {
        info!("Generating image for design: {}", id);
        image_generator.generate_image(&req.style, &req.prompt)
    }).await {
        Ok(Ok(image)) => {
            info!("Successfully generated image for design: {}", id);
            image
        },
        Ok(Err(e)) => {
            error!("Failed to generate image: {}", e);
            return HttpResponse::BadRequest().body("Failed to generate image");
        },
        Err(e) => {
            error!("Server error while generating image: {}", e);
            return HttpResponse::InternalServerError().body("Server error");
        },
    };

    let design = Design {
        image: base64_image.clone(),
        ..design
    };

    match web::block(move || {
        info!("Storing design {} in database", id);
        db.create_design(&design)
    }).await {
        Ok(Ok(_)) => {
            info!("Successfully created design: {}", id);
            HttpResponse::Ok().json(CreateDesignResponse {
                id,
                image: base64_image,
            })
        },
        Ok(Err(e)) => {
            error!("Failed to store design {}: {}", id, e);
            HttpResponse::InternalServerError().body("Failed to store design")
        },
        Err(e) => {
            error!("Server error while storing design {}: {}", id, e);
            HttpResponse::InternalServerError().body("Server error")
        },
    }
}

pub async fn get_design(
    db: web::Data<Database>,
    path: web::Path<String>,
) -> impl Responder {
    let path_str = path.into_inner();
    debug!("Attempting to parse design ID: {}", path_str);
    
    let id = match Uuid::parse_str(&path_str) {
        Ok(uuid) => uuid,
        Err(e) => {
            error!("Invalid design ID format '{}': {}", path_str, e);
            return HttpResponse::BadRequest().body("Invalid design ID");
        }
    };

    info!("Fetching design: {}", id);
    match db.get_design_by_id(&id) {
        Ok(Some(design)) => {
            info!("Successfully retrieved design: {}", id);
            HttpResponse::Ok().json(design)
        },
        Ok(None) => {
            debug!("Design not found: {}", id);
            HttpResponse::NotFound().body("Design not found")
        },
        Err(e) => {
            error!("Database error while fetching design {}: {}", id, e);
            HttpResponse::InternalServerError().body("Database error")
        },
    }
}

pub fn configure_routes(cfg: &mut web::ServiceConfig) {
    cfg.service(
        web::resource("/design")
            .route(web::post().to(create_design))
    )
    .service(
        web::resource("/design/{id}")
            .route(web::get().to(get_design))
    );
}