use rusqlite::{params, Result};
use chrono::NaiveDateTime;
use uuid::Uuid;
use std::sync::Arc;
use r2d2_sqlite::SqliteConnectionManager;
use r2d2::Pool;
use crate::models::Design;
use log::{info, error, debug};

pub struct Database {
    pool: Arc<Pool<SqliteConnectionManager>>,
}

impl Database {
    pub fn new(database_path: &str) -> Result<Self> {
        info!("Initializing database connection pool for: {}", database_path);
        let manager = SqliteConnectionManager::file(database_path);
        let pool = Pool::builder()
            .max_size(10)
            .build(manager)
            .expect("Failed to create pool");

        let conn = pool.get().expect("Failed to get connection from pool");
        debug!("Creating designs table if it doesn't exist");
        conn.execute(
            "CREATE TABLE IF NOT EXISTS designs (
                id TEXT PRIMARY KEY,
                style TEXT NOT NULL,
                prompt TEXT NOT NULL,
                image TEXT NOT NULL,
                created_at TEXT NOT NULL,
                user_id TEXT NOT NULL
            )",
            [],
        )?;

        info!("Database initialization complete");
        Ok(Self { 
            pool: Arc::new(pool)
        })
    }

    pub fn create_design(&self, design: &Design) -> Result<()> {
        debug!("Getting database connection for design creation");
        let conn = self.pool.get().expect("Failed to get connection");
        
        debug!("Inserting design {} into database", design.id);
        match conn.execute(
            "INSERT INTO designs (id, style, prompt, image, created_at, user_id) 
             VALUES (?1, ?2, ?3, ?4, ?5, ?6)",
            params![
                design.id.to_string(), 
                design.style, 
                design.prompt, 
                design.image, 
                design.created_at.to_string(), 
                design.user_id
            ],
        ) {
            Ok(_) => {
                info!("Successfully created design: {}", design.id);
                Ok(())
            },
            Err(e) => {
                error!("Failed to create design {}: {}", design.id, e);
                Err(e)
            }
        }
    }

    pub fn get_design_by_id(&self, id: &Uuid) -> Result<Option<Design>> {
        debug!("Getting database connection for design retrieval");
        let conn = self.pool.get().expect("Failed to get connection");
        
        debug!("Preparing query for design: {}", id);
        let mut stmt = conn.prepare(
            "SELECT id, style, prompt, image, created_at, user_id 
             FROM designs WHERE id = ?1"
        )?;

        debug!("Executing query for design: {}", id);
        let design_iter = stmt.query_map(params![id.to_string()], |row| {
            Ok(Design {
                id: Uuid::parse_str(&row.get::<_, String>(0)?).unwrap(),
                style: row.get(1)?,
                prompt: row.get(2)?,
                image: row.get(3)?,
                created_at: NaiveDateTime::parse_from_str(&row.get::<_, String>(4)?, "%Y-%m-%d %H:%M:%S%.f").unwrap(),
                user_id: row.get(5)?,
            })
        })?;

        match design_iter.collect::<Result<Vec<Design>, _>>() {
            Ok(designs) => {
                let design = designs.into_iter().next();
                if design.is_some() {
                    info!("Successfully retrieved design: {}", id);
                } else {
                    debug!("No design found with id: {}", id);
                }
                Ok(design)
            },
            Err(e) => {
                error!("Error retrieving design {}: {}", id, e);
                Err(e)
            }
        }
    }
}