use reqwest::blocking::Client;
use serde_json::json;
use thiserror::Error;
use log::{info, error, debug};

#[derive(Error, Debug)]
pub enum ImageGenerationError {
    #[error("Network error: {0}")]
    NetworkError(#[from] reqwest::Error),
    #[error("Invalid API response")]
    InvalidResponse,
}

pub struct ExternalImageGenerator {
    api_key: String,
    api_url: String,
    model: String,
}

const SIZE: &str = "1024x1024";
const NUM_IMAGES: i32 = 1;
// response_format: "b64_json" or "url"
const RESPONSE_FORMAT: &str = "url";

impl ExternalImageGenerator {
    pub fn new(api_key: String, api_url: String, model: String) -> Self {
        Self { api_key, api_url, model }
    }

    pub fn generate_image(&self, style: &str, prompt: &str) -> Result<String, ImageGenerationError> {
        debug!("Creating HTTP client for image generation");
        let client = Client::new();
        
        let formatted_prompt = format!("A painting of a {} in the style of {}", prompt, style);
        debug!("Formatted prompt: {}", formatted_prompt);

        let payload = json!({
            "model": self.model,
            "prompt": formatted_prompt,
            "n": NUM_IMAGES,
            "size": SIZE,
            "response_format": RESPONSE_FORMAT,
        });

        info!("Sending image generation request to external API");
        let response = match client.post(&self.api_url)
            .header("Content-Type", "application/json")
            .bearer_auth(self.api_key.clone())
            .json(&payload)
            .timeout(std::time::Duration::from_secs(60))
            .send() {
                Ok(resp) => resp,
                Err(e) => {
                    error!("Network error during image generation: {}", e);
                    return Err(ImageGenerationError::NetworkError(e));
                }
            };

        if response.status().is_success() {
            debug!("Successfully received response from image generation API");
            match response.json::<serde_json::Value>() {
                Ok(json) => {
                    let image_url = json["data"][0][RESPONSE_FORMAT].as_str().unwrap();
                    info!("Image generated successfully: {}", image_url);
                    Ok(image_url.to_string())
                },
                Err(e) => {
                    error!("Failed to parse API response: {}", e);
                    Err(ImageGenerationError::NetworkError(e))
                }
            }
        } else {
            error!("API returned error status: {}, error: {:?}", response.status(), response.text());
            Err(ImageGenerationError::InvalidResponse)
        }
    }
}