mod models;
mod database;
mod api;
mod external_image_generator;

use actix_web::{middleware::Logger, web, App, HttpServer};
use dotenv::dotenv;
use std::env;
use log::{info, warn};


#[actix_rt::main]
async fn main() -> std::io::Result<()> {
    dotenv().ok();

    let log_level = env::var("LOG_LEVEL").unwrap_or_else(|_| {
        warn!("LOG_LEVEL not set in .env, using default: info");
        "info".to_string()
    });
    
    env_logger::Builder::from_env(env_logger::Env::default().default_filter_or(&log_level))
        .format_timestamp_millis()
        .format_module_path(true)
        .init();

    info!("Starting application...");

    let server_addr = env::var("SERVER_ADDRESS").unwrap_or_else(|_| {
        warn!("SERVER_ADDRESS not set, using default: 127.0.0.1:8080");
        "127.0.0.1:8080".to_string()
    });

    let database_path = env::var("DATABASE_PATH").expect("DATABASE_PATH must be set");
    info!("Using database at: {}", database_path);

    let api_key = env::var("OPENAI_API_KEY").expect("OPENAI_API_KEY must be set");
    let api_url = env::var("OPENAI_API_URL").expect("OPENAI_API_URL must be set");
    let ai_model = env::var("OPENAI_MODEL").expect("OPENAI_MODEL must be set");
    info!("External API configured with URL: {} and model: {}", api_url, ai_model);

    let db = web::Data::new(
        database::Database::new(&database_path)
            .expect("Failed to create database connection")
    );
    info!("Database connection pool initialized");

    let image_generator = web::Data::new(
        external_image_generator::ExternalImageGenerator::new(
            api_key, 
            api_url,
            ai_model
        )
    );
    info!("Image generator service initialized");

    info!("Starting server on {}", server_addr);
    HttpServer::new(move || {
        App::new()
            .app_data(db.clone())
            .app_data(image_generator.clone())
            .configure(api::configure_routes)
            .wrap(Logger::default())
    })
    .bind(server_addr)?
    .run()
    .await
}